# Copyright 2016 GetWellNetwork, Inc., BSD copyright and disclaimer apply

import sys
import dbus
from dbus.mainloop.glib import DBusGMainLoop
from gi.repository import GObject

from bjarkan import find_adapter, get_managed_objects



def quit( mainloop ):
	mainloop.quit()



def scan_devices( duration = 10 ):
	""" This causes the bluetooth system to scan for any broadcasting devices. Once found, the devices get added to a
	dbus backed database specific for bluetooth for retrieval later.

	Args:
		duration (int): the amount of time that the scan should run for in seconds.
	"""
	DBusGMainLoop( set_as_default = True )
	bus = dbus.SystemBus()
	adapter = find_adapter()

	adapter.StartDiscovery()

	mainloop = GObject.MainLoop()
	GObject.timeout_add( duration * 1000, quit, mainloop )
	mainloop.run()



def gather_device_info():
	"""This function is responsible for digging through the dbus bluetooth database and retrieving all the known
	devices and the information about those devices. The database is seeded from scan_devices.

	Returns:
		List of device objects. Each object is one device and consisting of the properties of that device.
			[
				{
					'alias': 'Bluetooth Keyboard',
					'address': '00:00:00:00:00:00',
					'rssi': '-54',
					'icon': 'input-keyboard',
					'paired': 0,
					'connected': 0
				}
			]
	"""
	devices = []
	bus = dbus.SystemBus()
	objects = get_managed_objects()
	all_devices = ( str( path ) for path, interfaces in objects.items() if DEVICE_INTERFACE in interfaces )
	for path, ifaces in objects.items():
		if ADAPTER_INTERFACE not in ifaces:
			continue
		device_list = [ d for d in all_devices if d.startswith( path + '/' ) ]
		for dev_path in device_list:
			dev = objects[dev_path]
			properties = dev[DEVICE_INTERFACE]
			rssi = None
			icon = None
			if 'RSSI' in properties:
				rssi = int( properties['RSSI'] )
			if 'Icon' in properties:
				icon = str( properties['Icon'] )
			alias = properties['Alias']
			address = properties['Address']
			paired = properties['Paired']
			connected = properties['Connected']
			devices.append( { 'alias': str( alias ), 'address': str( address ), 'rssi': rssi, 'icon': icon, 'paired': int( paired ), 'connected': int( connected ) } )

	return devices



def connected_devices():
	"""Fetches the dbus bluetooth database and returns a list of devices that have a connected value of '1'

	Returns:
		List of device objects. Each object is one device and consisting of the properties of that device.
			[
				{
					'alias': 'Bluetooth Keyboard',
					'address': '00:00:00:00:00:00',
					'rssi': '-54',
					'icon': 'input-keyboard',
					'paired': 1,
					'connected': 1
				}
			]
	"""
	devices = gather_device_info()
	connected = []
	for device in devices:
		if device['connected']:
			connected.append( device )

	return connected



def paired_devices():
	"""Fetches the dbus bluetooth database and returns a list of devices that have a paired value of '1'

	Returns:
		List of device objects. Each object is one device and consisting of the properties of that device.
			[
				{
					'alias': 'Bluetooth Keyboard',
					'address': '00:00:00:00:00:00',
					'rssi': '-54',
					'icon': 'input-keyboard',
					'paired': 1,
					'connected': 0
				}
			]
	"""
	devices = gather_device_info()
	paired = []
	for device in devices:
		if device['paired']:
			paired.append( device )

	return paired


def all_devices():
	"""Forces a scan to populate the bluetooth dbus database, fetches the database information and returns it.

	Returns:
		List of device objects. Each object is one device and consisting of the properties of that device.
			[
				{
					'alias': 'Bluetooth Keyboard',
					'address': '00:00:00:00:00:00',
					'rssi': '-54',
					'icon': 'input-keyboard',
					'paired': 1,
					'connected': 0
				}
			]
	"""
	scan_devices()
	devices = gather_device_info()
	return devices



# Copyright 2016 GetWellNetwork, Inc., BSD copyright and disclaimer apply

import dbus
import bjarkan



SERVICE_NAME = 'org.bluez'
ADAPTER_INTERFACE = SERVICE_NAME + '.Adapter1'
DEVICE_INTERFACE = SERVICE_NAME + '.Device1'



class DeviceNotFound( Exception ):
	pass


class AdapterNotFound( Exception ):
	pass



def get_managed_objects():
	"""Fetch and return all things in the dbus bluetooth database

	Returns:
		All objects that are in the dbus dictionary and the properties that go along with those devices
	"""
	bus = dbus.SystemBus()
	manager = dbus.Interface(bus.get_object( SERVICE_NAME, '/' ), 'org.freedesktop.DBus.ObjectManager' )
	return manager.GetManagedObjects()



def find_adapter( pattern = None ):
	"""Find the adapter for this specific host.

	Args:
		pattern: the name of the bluetooth adapter

	Returns:
		Object that represents the bluetooth adapter installed in the host.
	"""
	return find_adapter_in_objects( get_managed_objects(), pattern )



def find_adapter_in_objects( objects, pattern = None ):
	"""Does the heavy lifting of sifting through the dbus bluetooth database to find the adapter
	that is installed and used on the host system.

	Args:
		objects: the current contents of the dbus bluetooth
		pattern: the name of the bluetooth adapter

	Returns:
		Object that represents the bluetooth adapter installed in the host.

	Raises:
		AdapterNotFound: bluetooth adapter was not found in the dbus bluetooth database.
	"""
	bus = dbus.SystemBus()
	for path, ifaces in objects.items():
		adapter = ifaces.get( ADAPTER_INTERFACE )
		if not adapter:
			continue
		# If pattern is None
		# or
		# If pattern is equal to the value of the 'Address' property of the assumed adapter
		# or
		# If the assumed adapter ends with the pattern provided i.e: hci0
		if not pattern or pattern == adapter['Address'] or path.endswith( pattern ):
			obj = bus.get_object( SERVICE_NAME, path )
			return dbus.Interface( obj, ADAPTER_INTERFACE )

	raise AdapterNotFound( 'Bluetooth adapter not found: {}'.format( pattern ) )



def find_device( device_address, adapter_pattern = None ):
	"""Attempt to find the device in the list of known devices in the dbus bluetooth database.

	Args:
		device_address: the mac address of the bluetooth device
		pattern: the name of the bluetooth adapter

	Returns:
		Object that represents the bluetooth adapter installed in the host.
	"""
	return find_device_in_objects( get_managed_objects(), device_address, adapter_pattern )



def find_device_in_objects( objects, device_address, adapter_pattern=None ):
	"""Does the heavy lifting of sifting through the dbus bluetooth database to match
	upon the device_address provided.

	Args:
		objects: the current contents of the dbus bluetooth
		device_address: the mac address of the bluetooth device
		pattern: the name of the bluetooth adapter

	Returns:
		Object that represents the bluetooth device.

	Raises:
		DeviceNotFound: bluetooth device was not found in the dbus bluetooth database.
	"""
	bus = dbus.SystemBus()
	path_prefix = ''
	if adapter_pattern:
		adapter = find_adapter_in_objects( objects, adapter_pattern )
		path_prefix = adapter.object_path
	for path, ifaces in objects.items():
		device = ifaces.get( DEVICE_INTERFACE )
		if not device:
			continue
		if device['Address'] == device_address and path.startswith( path_prefix ):
			obj = bus.get_object( SERVICE_NAME, path )
			return dbus.Interface( obj, DEVICE_INTERFACE )

	raise DeviceNotFound( 'Bluetooth device not found: {} {}'.format( device_address, adapter_pattern ) )



#!/usr/bin/python3

from setuptools import setup, find_packages

setup( name = 'bjarkan',
    version = '1.1.0',
    description = 'Bluetooth command line utility',
    long_description = open( 'README.rst' ).read(),
    keywords = 'bluez bluetooth cli',
    author = 'GetWellNetwork',
    author_email = 'willdeberry@gmail.com',
    maintainer = 'Will DeBerry',
    maintainer_email = 'willdeberry@gmail.com',
    license = 'Copyright 2016 GetWellNetwork, Inc., BSD copyright and disclaimer apply',
    url = 'https://github.com/willdeberry/bjarkan',
    download_url = 'https://github.com/willdeberry/bjarkan',
    packages = find_packages( exclude = ['contrib', 'docs', 'tests'] ),
    install_requires = [
        'pygobject>=3.18.2'
    ],
    entry_points = {
        'console_scripts': [
            'bjarkan = bjarkan.bjarkan:main'
        ],
    },
    classifiers = [
        'Programming Language :: Python',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators'
    ]
)
